import OpenSSL
import os
import re
import shutil
import tarfile


class UpgradeArchive:
    @staticmethod
    def __read_file(file_path, flags):
        with open(file_path, flags) as f:
            return f.read()

    @staticmethod
    def validate(public_key_path, archive_path, extract_path):
        if os.path.exists(extract_path):
            shutil.rmtree(extract_path)

        tarfile.open(archive_path).extractall(extract_path)
        UpgradeArchive.validate_directory_content(public_key_path, extract_path)

        shutil.rmtree(extract_path)

    @staticmethod
    def validate_directory_content(public_key_path, path):
        regex = re.compile(r'\.sign$')
        files = os.listdir(path)
        files = list(filter(lambda i: not regex.search(i), files))

        with open(os.path.join(path, public_key_path), 'rb') as f:
            key_data = f.read()

        for file in files:
            file_data = UpgradeArchive.__read_file(os.path.join(path, file), flags='rb')
            signature_data = UpgradeArchive.__read_file(os.path.join(path, file + ".sign"), flags='rb')

            x509 = OpenSSL.crypto.X509()
            x509.set_pubkey(OpenSSL.crypto.load_publickey(OpenSSL.crypto.FILETYPE_PEM, key_data))
            OpenSSL.crypto.verify(x509, signature_data, file_data, 'sha256')
